#  efield.jlプログラム
#  2次元運動のシミュレーション
#  電界中の荷電粒子のシミュレーションです
#　使い方　c:\>julia efield.jl

using Printf #  @printf()を使用

#グローバル変数
Q=(((0.0,0.0),10.0),((5.0,-5.0),5.0)) #電荷の位置と値
TIMELIMIT=20.0 #シミュレーション打ち切り時刻
RLIMIT=0.1 #距離rの最低値
H=0.01  #時刻の刻み幅

#main()関数
function main()
    t=0.0   #時刻t

    #係数の入力
    print("初速度v0xを入力してください:")
    vx = parse(Float64,readline())
    print("初速度v0yを入力してください:")
    vy = parse(Float64,readline())
    print("初期位置xを入力してください:")
    x = parse(Float64,readline())
    print("初期位置yを入力してください:")
    y = parse(Float64,readline())
    @printf("%-7.2f %-10.7f %-10.7f %-10.7f %-10.7f\n",t,x,y,vx,vy)
                                             #現在時刻と現在の位置

    #2次元運動の計算
    while t<TIMELIMIT  #打ち切り時間まで計算
        t=t+H #時刻の更新
        rmin=Inf #距離の最小値を初期化
        for qi in Q
            rx=qi[1][1]-x #距離rxの計算
            ry=qi[1][2]-y #距離ryの計算
            r=sqrt(rx*rx+ry*ry) #距離rの計算
            if r<rmin
                rmin=r #距離の最小値を更新
            end
            vx+=(rx/r/r/r*qi[2])*H #速度vxの計算
            vy+=(ry/r/r/r*qi[2])*H #速度vyの計算
        end
        x+=vx*H #位置xの計算
        y+=vy*H #位置yの計算
        @printf("%-7.2f %-10.7f %-10.7f %-10.7f %-10.7f\n",t,x,y,vx,vy)
                                                 #現在時刻と現在の位置
        if rmin<RLIMIT 
            break #電荷に非常に近づいたら終了
        end
    end
end

# main()関数の実行
main()

# efield.jlの終わり