#  gca1.jlプログラム
#  セルオートマトン（1次元）計算プログラム 
#　ルールと初期状態から、時間発展を計算する
#　使い方　c:\>julia gca1.jl

using Images , ImageView  #　描画

#グローバル変数
N = 256     # セルの最大個数
R = 8      # ルール表の大きさ
MAXT = 256  # 繰り返しの回数

#下請け関数の定義
# setrule()関数
function setrule(rule,ruleno)
    """ルール表の初期化"""
    # ルール表の書き込み
    for i = 1 : R
        rule[i] = ruleno % 2
        ruleno = ruleno ÷ 2  # 左シフト
    end
    # ルールの出力
    for i = R : -1 : 1
        println(rule[i])
    end
end
# setrule()関数の終わり

# initca()関数
function initca(ca)
    """セルオートマトンへの初期値の読み込み"""
    # 初期値を読み込む
    print("caの初期値を入力してください:")
    line = readline()
    println()
    #内部表現への変換
    for no = 1 : length(line)
        ca[no] = parse(Int , line[no])
    end
end
# initca()関数の終わり

# putca()関数
function putca(ca)
    """caの状態の出力"""
    for no = N : -1 : 1
        print(ca[no])
    end
    println()
end
# putca()関数の終わり

# nextt()関数
function nextt(ca,rule)
    """caの状態の更新"""
    nextca = zeros(Int , N)  # 次世代のca
    # ルールの適用
    for i = 2 : N - 1
        nextca[i] = rule[ca[i + 1] * 4 + ca[i] * 2 + ca[i - 1] + 1]
    end
    # caの更新
    ca .= nextca
end
# nextt()関数の終わり

#main()関数
function main()
    outputdata = zeros(Float64 , MAXT+1 , N) #出力用データ
    # ルール表の初期化
    rule = zeros(Int , R)  # ルール表の作成
    print("ルール番号を入力してください:")
    ruleno = parse(Int,readline())
    println()
    if ruleno < 0 || ruleno > 255
        println("ルール番号が正しくありません(", ruleno, ")")
        exit()
    end
    setrule(rule, ruleno)  # ルール表をセット
    # セルオートマトンへの初期値の読み込み
    ca =  zeros(Int , N) # セルの並び
    initca(ca)  # 初期値読み込み
    putca(ca)   # caの状態の出力
    for i = 1 : N
        outputdata[1,i] = ca[i]
    end 
    # 時間発展の計算
    for t = 1 : MAXT
        nextt(ca, rule)  # 次の時刻に更新
        putca(ca)        # caの状態の出力
        for i = 1 : N
            outputdata[t+1 , i] =ca[i] # 出力用データの更新
        end
    end
    # グラフ出力
    img = colorview(Gray,outputdata)
    imshow(img)
    readline()
end

# main()関数の実行
main()

#gca1.jlの終わり