#  glife.jlプログラム
#　ライフゲーム計算プログラム 
#　2次元セルオートマトンの一種である、ライフゲームのプログラム
#　結果をグラフ描画する
#　使い方　c:\>julia glife.jl  < (初期状態ファイル名)
#　　又は　c:\>type (初期状態ファイル名) | julia life.jl 

using Plots  #　グラフ描画

#グローバル変数
N = 50      # ライフゲームの世界の大きさ
MAXT = 50  # 繰り返しの回数

#下請け関数の定義
# gputworld()関数
function gputworld(world)
    """worldの状態の出力"""
    # 生命の存在する座標をプロット
    x = []
    y = []
    for i = 1 : N
        for j = 1 : N
            if world[i,j] == 1
                push!(y , i)
                push!(x , j)
            end
        end
    end   
    p = scatter(xlims=(0,N),ylims=(0,N)) #表示ウィンドウ作成
    p = scatter!(x , y) # プロット
    display(p) # 描画
    sleep(0.2) # 待ち合わせ
end
# gputworld()関数の終わり

# putworld()関数
function putworld(world)
    """worldの状態の出力"""
    # worldの出力
    for i = 1 : N
        for j = 1 : N
            print(world[i,j])
        end
        println()
    end
end
# putworld()関数の終わり

# initworld()関数
function initworld(world)
    """初期値の読み込み"""
    chrworld =readlines()
    # 内部表現への変換
    for i = 1 : N
        for j = 1 : N
            world[i , j] = parse(Int,chrworld[i][j])
        end
    end
end
# initworld()関数の終わり

# nextt()関数
function nextt(world)
    """worldの状態の更新"""
    nextworld =zeros(Int , N, N) # 次世代
    # ルールの適用
    for i = 2 : N-1
        for j = 2 : N - 1
            nextworld[i,j] = calcnext(world, i, j)
        end
    end
    # worldの更新
    world .= nextworld
end
# nextt()関数の終わり

# calcnext()関数
function calcnext(world, i, j)
    """1セルの状態の更新"""
    no_of_one = 0  # 周囲にある状態1のセルの数
    # 状態1のセルを数える
    for x = i - 1 : i + 1
        for y = j - 1 : j + 1
            no_of_one += world[x , y]
        end
    end
    no_of_one -= world[i , j]  # 自分自身はカウントしない
    # 状態の更新
    if no_of_one == 3
        return 1            # 誕生
    elseif no_of_one == 2
        return world[i , j]  # 存続
    else
        return 0  # 上記以外
    end
end
# calcnext()関数の終わり

#main()関数
function main()
    world =  zeros(Int , N, N)
    # world[][]への初期値の読み込み
    initworld(world)
    println("t=0")   # 初期時刻の出力
    #putworld(world)  # worldの状態の出力
    gputworld(world) # グラフ表示

    # 時間発展の計算
    for t = 1 : MAXT
        nextt(world)     # 次の時刻に更新
        println("t=", t) # 時刻の出力
        #putworld(world)  # worldの状態の出力
        gputworld(world) # グラフ表示
    end
end

# main()関数の実行
main()

#glife.jlの終わり