#  gtraffic.jlプログラム
#  セルオートマトンに基づく交通流シミュレーション
#　ルールと初期状態から、時間発展を計算する
#　使い方　c:\>julia gtraffic.jl

using Plots  #　グラフ描画

#グローバル変数
N = 50      # セルの最大個数
R = 8       # ルール表の大きさ
MAXT = 50   # 繰り返しの回数
RULE = 184  # ルール番号（184に固定）

#下請け関数の定義
# gputca()関数
function gputca(ca)
    """caの状態の出力"""
    x = []
    y = []
    for no = N : -1 : 1
        if ca[no] == 1
            push!(x , N - no + 1)
            push!(y , 1 )
        end
    end
    p = scatter(xlims=(0,N),ylims=(0,2)) #表示ウィンドウ作成
    p = scatter!(x , y) # プロット
    display(p) # 描画
    sleep(0.5) # 待ち合わせ
end
# gputca()関数の終わり

# setrule()関数
function setrule(rule,ruleno)
    """ルール表の初期化"""
    # ルール表の書き込み
    for i = 1 : R
        rule[i] = ruleno % 2
        ruleno = ruleno ÷ 2  # 左シフト
    end
end
# setrule()関数の終わり

# initca()関数
function initca(ca)
    """セルオートマトンへの初期値の読み込み"""
    # 初期値を読み込む
    print("caの初期値を入力してください:")
    line = readline()
    println()
    #内部表現への変換
    for no = 1 : length(line)
        ca[N - no] = parse(Int , line[no])
    end
end
# initca()関数の終わり

# putca()関数
function putca(ca)
    """caの状態の出力"""
    for no = N : -1 : 1
        if ca[no] == 1
            print('-')
        else
            print(' ')
        end
    end
    println()
end
# putca()関数の終わり

# nextt()関数
function nextt(ca,rule)
    """caの状態の更新"""
    nextca = zeros(Int , N)  # 次世代のca
    # ルールの適用
    for i = 2 : N - 1
        nextca[i] = rule[ca[i + 1] * 4 + ca[i] * 2 + ca[i - 1] + 1]
    end
    # caの更新
    ca .= nextca
end
# nextt()関数の終わり

#main()関数
function main()
    # ルール表の初期化
    print("流入率を入力してください:")
    flowrate = parse(Int,readline())
    println()
    if flowrate <= 0
        println("流入率が正しくありません(" , flowrate , ")")
        exit()
    end
    rule = zeros(Int , R)  # ルール表の作成
    setrule(rule, RULE)  # ルール表をセット
    # セルオートマトンへの初期値の読み込み
    ca =  zeros(Int , N) # セルの並び
    initca(ca)  # 初期値読み込み

    # 時間発展の計算
    for t = 1 : MAXT
        nextt(ca, rule)  # 次の時刻に更新
        if (t % flowrate) == 0 
            ca[N] = 1  # 自動車の流入
        end
        print("t=", t, "\t") 
        putca(ca)      # caの状態の出力
        gputca(ca)      # caの状態の出力
    end
end

# main()関数の実行
main()

#gtraffic.jlの終わり