#  ginfection.jlプログラム
#  ”感染”のエージェントシミュレーション
#   2次元平面内で動作するエージェント群
#   2種類のエージェントが相互作用します
#   結果をグラフ描画します
#　使い方　c:\>julia ginfection.jl

using Random # Randomモジュールを利用
using Plots  #　グラフ描画

#グローバル変数
N = 100           # エージェントの個数
TIMELIMIT = 100  # シミュレーション打ち切り時刻
SEED = 65535     # 乱数の種
R = 0.5      #近隣を規定する数値
W = 40     # 表示領域の大きさ

#Agent型の定義
mutable struct Agent
    category::Int #カテゴリ
    x::Float64    #x座標
    y::Float64    #y座標
end

#下請け関数の定義
#gput()関数
function gput(as)
    """状態のグラフ出力"""
    x0 = []
    y0 = []
    x1 = []
    y1 = []
    for a in as
        if a.category == 0
            push!(x0 , a.x)
            push!(y0 , a.y)
        elseif a.category == 1
            push!(x1 , a.x)
            push!(y1 , a.y)
        end
    end
    p = scatter(xlims=(-W,W),ylims=(-W,W)) #表示ウィンドウ作成
    p = scatter!(x0 , y0) # カテゴリ0をプロット
    p = scatter!(x1 , y1, markershape=:+) # カテゴリ1をプロット
    display(p) # 描画
    sleep(0.2) # 待ち合わせ
end
#gput()関数の終わり

#calcnext()関数
function calcnext(a,as,factor)
    """次時刻の状態の計算"""
    if a.category == 0
        cat0(a , as) # カテゴリ0の計算
    elseif a.category == 1
        cat1(a,factor) # カテゴリ1の計算
    else # 合致するカテゴリがない
        print("ERROR カテゴリがありません\n")
    end
end 
#calcnext()関数の終わり

#cat0()関数
function cat0(a , as)
    """カテゴリ0の計算"""
    # カテゴリ1のエージェントとの距離を調べる
    for ai in as
        if ai.category == 1
            c0x = a.x
            c0y = a.y
            ax = ai.x
            ay = ai.y
            if ((c0x - ax) * (c0x - ax) + (c0y - ay) * (c0y - ay)) < R
                # 隣接してカテゴリ1のエージェントがいる
                a.category = 1  # カテゴリ1に変身
            end
        end
    end
    # 位置の更新
    a.x += rand() - 0.5
    a.y += rand() - 0.5
end 
#cat0()関数の終わり

#cat1()関数
function cat1(a,factor)
    """カテゴリ1の計算"""
    a.x += (rand() - 0.5) * factor
    a.y += (rand() - 0.5) * factor
end
#cat1()関数の終わり

#putstate()関数
function putstate(a)
    """状態の出力"""
    println(a.category," ",a.x," ",a.y)
end
#putstate()関数の終わり

# calcn()関数
function calcn(as,factor)
    """次時刻の状態を計算"""
    for a in as
        calcnext(a,as,factor)
        putstate(a)
    end
end
# calcn()関数の終わり

#main()関数
function main()
    #初期化
    Random.seed!(SEED)
    # カテゴリ0のエージェントを生成
    as = [Agent(0,(rand() - 0.5) * 20,(rand() - 0.5) * 20) for i =1 : N] 
    # カテゴリ1のエージェントの設定
    as[1].category = 1
    as[1].x = -2 
    as[1].y = -2
    #カテゴリ１のエージェントの歩幅factorの設定
    print("カテゴリ１の歩幅factorを入力してください:")
    factor =parse(Float64,readline())
 
    # エージェントシミュレーション
    for t = 1 : TIMELIMIT
        println("t=" , t)
        calcn(as,factor)  # 次時刻の状態を計算
        gput(as)   # グラフ出力
    end
end

# main()関数の実行
main()
sleep(10) # 待ち合わせ

#ginfection.jlの終わり