#  gsa1.jlプログラム
#  シンプルなエージェントシミュレーション
#  2次元平面内で動作するエージェント
#  複数のエージェントがランダムウォークする
#  結果をグラフ描画する
#　使い方　c:\>julia gsa1.jl

using Random # Randomモジュールを利用
using Plots  #　グラフ描画

#グローバル変数
N = 100           # エージェントの個数
TIMELIMIT = 100  # シミュレーション打ち切り時刻
SEED = 65535     # 乱数の種
W = 20           # 表示領域の大きさ

#Agent型の定義
mutable struct Agent
    category::Int #カテゴリ
    x::Float64    #x座標
    y::Float64    #y座標
end

#下請け関数の定義
#gput()関数
function gput(as)
    """状態のグラフ出力"""
    x = []
    y = []
    for a in as
        push!(x , a.x)
        push!(y , a.y)
    end
    p = scatter(xlims=(-W,W),ylims=(-W,W)) #表示ウィンドウ作成
    p = scatter!(x , y) # プロット
    display(p) # 描画
    sleep(0.2) # 待ち合わせ
end
#gput()関数の終わり

#calcnext()関数
function calcnext(a)
    """次時刻の状態の計算"""
    if a.category == 0
        cat0(a) # カテゴリ0の計算
    else # 合致するカテゴリがない
        print("ERROR カテゴリがありません\n")
    end
end 
#calcnext()関数の終わり

#cat0()関数
function cat0(a)
    """カテゴリ0の計算"""
    # 次の座標を乱数によって決定
    a.x += (rand() - 0.5) * 2
    a.y += (rand() - 0.5) * 2
end 
#cat0()関数の終わり

#putstate()関数
function putstate(a)
    """状態の出力"""
    println(a.x," ",a.y)
end
#putstate()関数の終わり

# calcn()関数
function calcn(as)
    """次時刻の状態を計算"""
    for a in as
        calcnext(a)
        putstate(a)
    end
end
# calcn()関数の終わり

#main()関数
function main()
    #初期化
    Random.seed!(SEED)
    as = [Agent(0,0,0) for i =1 : N] # カテゴリ0のエージェントを生成
    
    # エージェントシミュレーション
    for t = 1 : TIMELIMIT
        println("t=" , t)
        calcn(as)  # 次時刻の状態を計算
        gput(as)   # グラフ出力
    end
end

# main()関数の実行
main()

#gsa1.jlの終わり