#  sa2.jlプログラム
#  シンプルなエージェントシミュレーション
#  2次元平面内で動作するエージェント
#   2種類のエージェントが相互作用します
#　使い方　c:\>julia sa2.jl

using Random # Randomモジュールを利用

#グローバル変数
N = 30           # エージェントの個数
TIMELIMIT = 100  # シミュレーション打ち切り時刻
SEED = 65535     # 乱数の種
R=0.1  #近隣を規定する数値
DX=0.1 #カテゴリ1のエージェントの速度
DY=0.1 #カテゴリ1のエージェントの速度

#Agent型の定義
mutable struct Agent
    category::Int #カテゴリ
    x::Float64    #x座標
    y::Float64    #y座標
end

#下請け関数の定義
#calcnext()関数
function calcnext(a , as)
    """次時刻の状態の計算"""
    if a.category == 0
        cat0(a , as) # カテゴリ0の計算
    elseif a.category == 1
        cat1(a , as) # カテゴリ1の計算
    else # 合致するカテゴリがない
        print("ERROR カテゴリがありません\n")
    end
end 
#calcnext()関数の終わり

#cat0()関数
function cat0(a , as)
    """カテゴリ0の計算"""
    # カテゴリ1のエージェントとの距離を調べる
    for ai in as
        if ai.category == 1
            c0x = a.x
            c0y = a.y
            ax = ai.x
            ay = ai.y
            if ((c0x - ax) * (c0x - ax) + (c0y - ay) * (c0y - ay)) < R
                # 隣接してカテゴリ1のエージェントがいる
                a.category = 1  # カテゴリ1に変身
            else  # カテゴリ1は近隣にはいない
                a.x += rand() - 0.5
                a.y += rand() - 0.5
            end
        end
     end
end 
#cat0()関数の終わり

#cat1()関数
function cat1(a , as)
    """カテゴリ1の計算"""
    a.x += DX
    a.y += DY
end
#cat1()関数の終わり

#putstate()関数
function putstate(a)
    """状態の出力"""
    println(a.category," ",a.x," ",a.y)
end
#putstate()関数の終わり

# calcn()関数
function calcn(as)
    """次時刻の状態を計算"""
    for a in as
        calcnext(a ,as)
        putstate(a)
    end
end
# calcn()関数の終わり

#main()関数
function main()
    #初期化
    Random.seed!(SEED)
    as = [Agent(0,0,0) for i =1 : N] # カテゴリ0のエージェントを生成
    # カテゴリ1のエージェントの設定
    as[1].category = 1
    as[1].x = -2 
    as[1].y = -2
 
    # エージェントシミュレーション
    for t = 1 : TIMELIMIT
        println("t=" , t)
        calcn(as)  # 次時刻の状態を計算
    end
end

# main()関数の実行
main()

#sa2.jlの終わり